{ DimmTimer
  Dimmer with Alarm Clock
  03.03.07
  C. Eberspcher

  This is an alarm clock, that wakes you by a light, that will start glowing and glow brighter
  until wake time with 100% brightness.
  That should give a friendly dawn effect.
  A beeper will beep at alarm time.
  The user interface is a button switch, a potentiometer and a lcd 2*16 display.
  There are 3 wake times, one for working days, one for saturday and one for sunday.

  The light dimmer works as many dimmers by varying the phase angle of mains power.
  pin 4 (TRIGOUT) drives the basis of a bc107 npn transistor over 2.2k resistor. Its emitter is connected to GND.
  The collector drives the input of a COSMO KSD210AC3-1 solid state relay (type without internal zero crossing switch).
  A lamp is connected to this solid state relay.

  The phase output has to be aligned to the phase of mains power.
  This is done by an alternating voltage from a transformator connected over a high resistor to pin4 (pllin).

  In the main display mode, the potentiometer will vary the brightness of the lamp.
  When the button is pressed you come to the main menue.

  The SetTime fuction is left with potentioneter 0.
  1 resets seconds
  2 advance one minute
  3 advance an hour
  4 advance a day (0=monday)

   
  


        cogs:
        0:    main/user interface
        1:    rc potentiometer input
        2:    clock
        3:    PLL output
        4:    mains trigger input
}  



CON
  _CLKMODE      = XTAL1 + PLL16X                        
  _XINFREQ      = 5_000_000

  RCMAX = 20_000  ' maximum RC Value using 50k Poti 0.1F
  
  MAXIPOS       = 11            ' Maximum Poti value

  BEEPTIME = 3                  ' seconds




'Ports:
  LED   =       27              'LED- PIN
  BUTTON =      2               ' Button Switch with pullup resistor
  POTI  =       1               ' POTI for RC
  BEEPER=       17              ' A beeper
    
{    ' LCD-Object 
  RS = 14                   
  RW = 26       'Ist auf Masse gelegt!!!                    
  E  = 15

  DB4 = 8
  DB7 = 11

  'SynthPLL:
    PLLOUT =      16 '
    TRIGOUT =     3
    PLLIN   =     4 '
    LED     =     27 
  }


  
  

VAR
    long RCValue, oldpos
    long seconds[4], minutes[4], hours[4], days[4], dT, T, TStack[30]
    long dawntime, dimvalue, tbright
    byte dimmon
    
OBJ
  LCD : "LCD_16X2_4BIT"
  RC  : "RCTime"
  PLL : "SynthPLL"
  
PUB main
    dira[LED]~~                 'Output
    dira[BEEPER]~~

    repeat 10                   ' Blinking LED for start
      !outa[LED]
      waitcnt(10_000_000 + cnt)

  LCD.START
  LCD.START

  'Initialize RCValue
    RCValue := 0
    RC.start(POTI,1,@RCValue )    'POTI Input
    
    cognew(TimeCounter,@TStack)

  'Initialize PLL
    'PLL.Synth("A",20,50)   'Start freq output - only for tests
    PLL.StartPLL                  

  'Start values
    days[0]:=    4
    hours[0]:=   19                ' actual time
    minutes[0]:= 50
    seconds[0]:= 0



    
    hours[1]:=   6                'work day alarm
    minutes[1]:= 0
    
    hours[2]:=   7                'saturday alarm
    minutes[1]:= 30

    hours[3]:=   8                'sunday alarm
    minutes[1]:= 0
        
    dawntime:= 20*60             '20 Minutes
    dimmon:= TRUE
    tbright:=     30*60          ' 30 minutes bright time

    
    {hours[0]:=   5                ' Test values
    minutes[0]:= 59
    seconds[0]:= 45
    dawntime:= 10             
    dimmon:= TRUE
    tbright:=     10}          
    
  MainLoop            



PUB BPressed                           'True, if button pressed, else false
  if ina[BUTTON]==0
    result:=TRUE
  else
    result:= FALSE      

PUB BRCPos(maxim)                      ' Pos of poti if button pressed, else old value
  if ina[BUTTON]==0
    result:=oldpos:=RCPos(maxim)
  else
    result:=oldpos

PUB RCPos(maxi) | help                 ' reads poti, values 0...maxi
  help:= RCVALUE <# RCMAX
  result:= maxi*help/RCMAX

PUB SetBrightTime                        ' User interface to set bright time in minutes
  repeat until BPressed
    LCD.CLEAR         
    'DispTime(i)
    
    LCD.STR(STRING("Set Bright Minutes:"))
    LCD.MOVE(1,2) 
    LCD.DEC (RCPos(100))
    
    if ina[BUTTON]==0           ' Button is pressed
      tbright:= 60*RCPos(100)

    WAITCNT(10_000_000 + CNT )  

PUB SetDawnTime                        ' User interface to set dawn time in minutes
  repeat until BPressed
    LCD.CLEAR         
    'DispTime(i)
    
    LCD.STR(STRING("Set Dawn Minutes:"))
    LCD.MOVE(1,2) 
    LCD.DEC (RCPos(100))
    
    if ina[BUTTON]==0           ' Button is pressed
      dawntime:= 60*RCPos(100)

    WAITCNT(10_000_000 + CNT )



PUB CheckDim | deltat                  ' gives back dim value as pc if time is in dawntime, else 0, activates beeper

    case days[0]
      0..4: deltat:= TimeDif(1,0)                'work day alarm time - actual time
      5:    deltat:= TimeDif(2,0)                'saturday alarm time - actual time
      6:    deltat:= TimeDif(3,0)                'sunday alarm time - actual time
           
    if deltat>-tbright and deltat<dawntime
      result:= 100-100*deltat/dawntime
      if result>99
        result:=99
    else
      result:=0

    if deltat => 0 and deltat < BEEPTIME
        outa[BEEPER]:=1
    else
        outa[BEEPER]:=0  
        
       


PUB TimeDif(ia, ib)                    ' gives back difference of timea- timeb in sec
    result:= seconds[ia]-seconds[ib]+60*(minutes[ia]-minutes[ib])
    result+= 3600*(hours[ia]-hours[ib])  
      
PUB SetTime(i)                         ' User interface to set times
  repeat until RCPos(MAXIPOS)==0
    LCD.CLEAR         
    DispTime(i)
    LCD.MOVE(1,2)     
    case i
      0: LCD.STR(STRING("Set Time:"))
      1: LCD.STR(STRING("Set Al Wrk:"))
      2: LCD.STR(STRING("Set Al Sa:"))
      3: LCD.STR(STRING("Set Al So:"))
    LCD.DEC (RCPos(MAXIPOS))
    if ina[BUTTON]==0           ' Button is pressed
      if RCPos(MAXIPOS) == 1
        if seconds<31
          seconds[i]:=0
        else
          Repeat (60-Seconds[i])
            NextSec(i)  
      if RCPos(MAXIPOS) == 2
        repeat 60
          NextSec(i)
      if RCPos(MAXIPOS) == 3
        repeat 3600      
          NextSec(i)
      if RCPos(MAXIPOS) == 4
        repeat 3600*24
          NextSec(i)
    WAITCNT(10_000_000 + CNT )            
          
PUB TimeCounter                        'Clock runs in its own cog
    dT := clkfreq
    T  := cnt
    repeat    
       T += dT
       waitcnt(T)
       NextSec(0)

PUB NextSec(i)                         'counts up seconds, minutes, hours, day
       seconds[i] ++
       if seconds[i] ==60
          seconds[i]:=0
          minutes[i] ++
          if minutes[i] == 60
             minutes[i] := 0 

             hours[i] ++
             if hours[i] == 24
                hours[i] := 0

                days[i] ++
                if days[i]==7
                   days[i]:=0

PUB DispTime(i)                        'displays day and time on LCD first line left
    LCD.MOVE(1,1)
    LCD.STR(STRING(" >  :  :  "))
    LCD.MOVE(1,1) 
    LCD.DEC(days[i])
    LCD.MOVE(3,1) 
    LCD.DEC(hours[i])
    LCD.MOVE(6,1) 
    LCD.DEC(minutes[i])
    LCD.MOVE(9,1) 
    LCD.DEC(seconds[i])
    

                   
    
PUB MainLoop | oldsec, oldppos         'Main Loop Displays Time and Selects Dimm 
  
  LCD.MOVE(1,1)
  LCD.STR(STRING("DimmTimer 1.0CWE"))

  WAITCNT(120_000_000 + CNT)

  Repeat
     'repeat while (oldsec==seconds)
    WAITCNT(10_000_000 + CNT )
    if seconds<>oldsec
        oldsec:= seconds    
        LCD.CLEAR
        DispTime(0)                   ' Show Time

        if dimmon                    ' Alarm activated ?
          LCD.STR(STRING(" Al "))
          dimvalue:= CheckDim
        else
          LCD.STR(STRING(" NA "))
          dimvalue:=0   
                   
        LCD.DEC(dimvalue)

        LCD.MOVE(1,2)

        LCD.DEC(CLKFREQ*10/(PLL.GetFRQ/10))
        oldppos:=-1

    if not dimvalue
      if RCPOS(110)<>oldppos
        oldppos:=RCPOS(110)
        LCD.MOVE(11,2)
        LCD.STR(STRING("P:    "))
        LCD.MOVE(13,2)
        LCD.DEC (RCPos(110))
        if RCPos(MAXIPOS)<11
          PLL.SetPC(RCPos(110))
          'PLL.SetPC(18) 
          'if RCPos(MAXIPOS)>10
    else
      LCD.MOVE(11,2)
      LCD.STR(STRING("Dimmer"))
      PLL.SetPC(dimvalue)
         
    if BPressed
        Menue


PUB Menue | Select                     ' Main Menue
  LCD.CLEAR
  LCD.MOVE(1,1)  '1234567890123456
  LCD.STR(STRING("0Rt1of2on3Ti4Wr"))
  LCD.MOVE(1,2)
  LCD.STR(STRING("5Sa6So7DT8BT"))

  repeat while BPressed

  Select:= oldpos :=11  
  repeat until Select<11
    LCD.MOVE(13,2)
    LCD.STR(STRING("    "))
    LCD.MOVE(14,2)
    LCD.DEC(RCPOS(MAXIPOS))
     
    Select:=BRCPos(MAXIPOS)
    WAITCNT(20_000_000 + CNT )    
    
    
  repeat while BPressed
   
  CASE Select
    0: 'Return to main loop
    1: dimmon:= FALSE
    2: dimmon:= TRUE
    3: SetTime(0)
    4: SetTime(1)
    5: SetTime(2)
    6: SetTime(3)
    7: SetDawnTime
    8: SetBrightTime
    
    
    